//This file is adapted from https://github.com/ArduCAM/Arduino with MIT license
#ifndef P2_CAMERA
#define P2_CAMERA

#include "ov2640_regs.h"
#include "ov7675_regs.h"
#include "ov5640_regs.h"

//Translate Aruduino...
#define pgm_read_word(x)        ( ((*((unsigned char *)x + 1)) << 8) + (*((unsigned char *)x)))

/****************************************************/
/* Sensor related definition 												*/
/****************************************************/
#define BMP 	0
#define JPEG	1
#define RAW	  2

int DeviceAddress;// (0x78) //(0x60)//(0x42) //(0x60) //i2c address for camera
//0x60 for ov2640
//0x42 for ov7675
//0x78 for ov5640

#define OV7670		0	
#define MT9D111_A	1
#define OV7675		2
#define OV5642		3
#define OV3640  	4
#define OV2640  	5
#define OV9655		6
#define MT9M112		7
#define OV7725		8
#define OV7660		9
#define MT9M001 	10
#define OV5640 		11
#define MT9D111_B	12
#define OV9650		13
#define MT9V111		14
#define MT9T112		15
#define MT9D112		16
#define MT9V034 	17
#define MT9M034     18

i2c_Port* pI2C1; //i2c port to use

void CamSetI2cPort(i2c_Port* p)
{//Specify i2c port to use
	pI2C1 =p;
}

/****************************************************************/
/* define a structure for sensor register initialization values */
/****************************************************************/
struct sensor_reg {
	uint16_t reg;
	uint16_t val;
};

void ProbeCamera(int Type, i2c_Port* pI2C)
{//See if camera ID matches settings over I2C
	uint8_t buf1 = 1;
	uint8_t buf2;
	uint16_t ID;
	if (Type == OV2640)
	{
		i2c_WriteRegisters(pI2C, 0x60, 0xFF, 1, &buf1);
		i2c_ReadRegisters(pI2C, 0x60, OV2640_CHIPID_HIGH, 1, &buf1);
		i2c_ReadRegisters(pI2C, 0x60, OV2640_CHIPID_LOW, 1, &buf2);
		ID = (buf1 << 8) + buf2;
		printf("Camera ID = 0x%x\n", ID);
	}
	else if (Type == OV7675)
	{
		i2c_ReadRegisters(pI2C, 0x42, OV7675_CHIPID_HIGH, 1, &buf1);
		i2c_ReadRegisters(pI2C, 0x42, OV7675_CHIPID_LOW, 1, &buf2);
		ID = (buf1 << 8) + buf2;
		printf("Camera ID = 0x%x\n", ID);
	}
	else if (Type == OV5640)
	{
		i2c_ReadRegisters16(pI2C, 0x78, OV5640_CHIPID_HIGH, 1, &buf1);
		i2c_ReadRegisters16(pI2C, 0x78, OV5640_CHIPID_LOW, 1, &buf2);
		ID = (buf1 << 8) + buf2;
		printf("Camera ID = 0x%x\n", ID);
	}
}

void InitCamera(int Xclk, int PixelClockFreq, int RST, int Type, i2c_Port* pI2C, int m_fmt)  //RJA added format option 29Dec22
{
	_wrpin(Xclk, 0b1001100);
	_wxpin(Xclk, 1);
	int x;
	int freq = 20000000;    //Camera clock frequency  
	int p2clk = _clkfreq;
	__asm
	{//thank you Ozpropdev!!!    (From RJO__) 
		qfrac	freq, p2clk
		getqx   x
	}
	_wypin(Xclk, x);// period);
	_pinh(Xclk);
	_waitms(1000);

	//twist reset
	_pinl(RST);
	_waitms(3);
	_pinh(RST);
	_waitms(2);

	//Initialize the camera using Arducam Code
	CamSetI2cPort(pI2C);
	//int m_fmt = BMP;

	//int DeviceAddress;// (0x78) //(0x60)//(0x42) //(0x60) //i2c address for camera
	//0x60 for ov2640
	//0x42 for ov7675
	//0x78 for ov5640

	//Scope measurements 31Dec22:
	//in QVGA raw mode, pixel clock is 15 MHz
	//In QVGA jpeg mode, pixel clock is 10 MHz, capture data on rising edge
	//In VGA jpeg mode, pixel clock is 20 MHz, capture data on rising edge
	//in SVGA jpeg mode, pixel clock is 15 MHz, captur data on rising edge
	//in XGA jpeg mode, pixel clock is 15 MHz, captur data on rising edge
	//In 1600x1200 jpeg mode, pixel clock is 20 MHz, capture data on rising edge

	if (Type == OV2640)
	{
		DeviceAddress = 0x60;
		wrSensorReg8_8(0xff, 0x01);
		wrSensorReg8_8(0x12, 0x80);
		_waitms(100);
		if (m_fmt == JPEG)
		{
			wrSensorRegs8_8(OV2640_JPEG_INIT);
			wrSensorRegs8_8(OV2640_YUV422);
			wrSensorRegs8_8(OV2640_JPEG);
			wrSensorReg8_8(0xff, 0x01);
			wrSensorReg8_8(0x15, 0x00);
			//wrSensorReg8_8(0x15, 0x20); //RJA wants pixel clock qualified by hsync (doesn't seem to work...)
			//wrSensorRegs8_8(OV2640_320x240_JPEG);  //Works with P2 clock at 200 MHz, not 300 MHz, no waitx
			//wrSensorRegs8_8(OV2640_640x480_JPEG);  //This one needs a waitx in the loop that depends on clock frequency, 10 for 300 mhz, 5 for 250 Mhz...  Why?
			//wrSensorRegs8_8(OV2640_800x600_JPEG); //Seem to need 300 MHz clock starting here, no waitx
			//wrSensorRegs8_8(OV2640_1024x768_JPEG);  
			wrSensorRegs8_8(OV2640_1600x1200_JPEG);
			//wrSensorReg8_8(0xff, 0x00);
			//wrSensorReg8_8(0x44, 0x32); //0x44 is compression setting 10 (default) is highest, 63 is lowest
		}
		else
		{
			wrSensorRegs8_8(OV2640_QVGA);
		}
	}
	else if (Type == OV7675)
	{
		DeviceAddress = 0x42;
		wrSensorReg8_8(0x12, 0x80);
		_waitms(100);
		wrSensorRegs8_8(OV7675_QVGA);
		printf("OV7675 camera initilized.\n");
	}
	else if (Type == OV5640)
	{
		DeviceAddress = 0x78;
		_waitms(100);
		if (m_fmt == JPEG)
		{
			wrSensorReg16_8(0x3103, 0x11);
			wrSensorReg16_8(0x3008, 0x82);
			_waitms(100);
			wrSensorRegs16_8(OV5640YUV_Sensor_Dvp_Init);
			_waitms(500);
			wrSensorRegs16_8(OV5640_JPEG_QSXGA);
			wrSensorRegs16_8(OV5640_QSXGA2QVGA);
	#if (defined(OV5640_MINI_5MP_PLUS) || (defined ARDUCAM_SHIELD_V2))
			wrSensorReg16_8(0x4407, 0x04);
	#else
			wrSensorReg16_8(0x4407, 0x0C);
	#endif
		}
		else
		{
			printf("OV5640 QVGA init.\n");
			wrSensorReg16_8(0x3103, 0x11);
			wrSensorReg16_8(0x3008, 0x82);
			_waitms(500);
			wrSensorRegs16_8(OV5640YUV_Sensor_Dvp_Init);
			_waitms(5);
			wrSensorRegs16_8(OV5640_RGB_QVGA);
			//wrSensorRegs16_8(OV5640_Auto_Focus);  //RJA works?

		}
	}
}

void wrSensorReg8_8(uint8_t regID, uint8_t regDat)
{// Read/write 8 bit value to/from 8 bit register address	
	i2c_WriteRegisters(pI2C1, DeviceAddress, regID, 1, &regDat);
}


int wrSensorRegs8_8(struct sensor_reg reglist[])
{// Write 8 bit values to 8 bit register address

	uint16_t reg_addr = 0;
	uint16_t reg_val = 0;
	const struct sensor_reg* next = reglist;
	while ((reg_addr != 0xff) | (reg_val != 0xff))
	{
		reg_addr = pgm_read_word(&next->reg);
		reg_val = pgm_read_word(&next->val);
		wrSensorReg8_8(reg_addr, reg_val);
		next++;

	}

	return 1;
}

// Read/write 8 bit value to/from 16 bit register address
void wrSensorReg16_8(int regID, int regDat)
{
	i2c_WriteRegisters16(pI2C1, DeviceAddress, regID, 1, (uint8_t*) &regDat);
}

// Write 8 bit values to 16 bit register address
int wrSensorRegs16_8(const struct sensor_reg reglist[])
{
	unsigned int reg_addr=0;
	unsigned char reg_val=0;
	const struct sensor_reg* next = reglist;

	while ((reg_addr != 0xffff) | (reg_val != 0xff))
	{
		reg_addr = pgm_read_word(&next->reg);
		reg_val = pgm_read_word(&next->val);

		wrSensorReg16_8(reg_addr, reg_val);
		next++;

	}
	return 1;
}




#define OV2640_160x120 		0	//160x120
#define OV2640_176x144 		1	//176x144
#define OV2640_320x240 		2	//320x240
#define OV2640_352x288 		3	//352x288
#define OV2640_640x480		4	//640x480
#define OV2640_800x600 		5	//800x600
#define OV2640_1024x768		6	//1024x768
#define OV2640_1280x1024	7	//1280x1024
#define OV2640_1600x1200	8	//1600x1200



#define OV3640_176x144 		0	//176x144
#define OV3640_320x240 		1	//320x240
#define OV3640_352x288 		2	//352x288
#define OV3640_640x480		3	//640x480
#define OV3640_800x600 		4	//800x600
#define OV3640_1024x768		5 //1024x768
#define OV3640_1280x960	  6	//1280x960
#define OV3640_1600x1200	7	//1600x1200
#define OV3640_2048x1536	8	//2048x1536


#define OV5642_320x240 		0	//320x240
#define OV5642_640x480		1	//640x480
#define OV5642_1024x768		2	//1024x768
#define OV5642_1280x960 	3	//1280x960
#define OV5642_1600x1200	4	//1600x1200
#define OV5642_2048x1536	5	//2048x1536
#define OV5642_2592x1944	6	//2592x1944
#define OV5642_1920x1080  7


#define OV5640_320x240 		0	//320x240 
#define OV5640_352x288		1	//352x288
#define OV5640_640x480 	  2	//640x480
#define OV5640_800x480	  3	//800x480
#define OV5640_1024x768	  4	//1024x768
#define OV5640_1280x960	  5	//1280x960	
#define OV5640_1600x1200	6	 //1600x1200
#define OV5640_2048x1536	7  //2048x1536
#define OV5640_2592x1944	8	 //2592x1944



//Light Mode

#define Auto                 0
#define Sunny                1
#define Cloudy               2
#define Office               3
#define Home                 4

#define Advanced_AWB         0
#define Simple_AWB           1
#define Manual_day           2
#define Manual_A             3
#define Manual_cwf           4
#define Manual_cloudy        5



//Color Saturation 

#define Saturation4          0
#define Saturation3          1
#define Saturation2          2
#define Saturation1          3
#define Saturation0          4
#define Saturation_1         5
#define Saturation_2         6
#define Saturation_3         7
#define Saturation_4         8

//Brightness

#define Brightness4          0
#define Brightness3          1
#define Brightness2          2
#define Brightness1          3
#define Brightness0          4
#define Brightness_1         5
#define Brightness_2         6
#define Brightness_3         7
#define Brightness_4         8


//Contrast

#define Contrast4            0
#define Contrast3            1
#define Contrast2            2
#define Contrast1            3
#define Contrast0            4
#define Contrast_1           5
#define Contrast_2           6
#define Contrast_3           7
#define Contrast_4           8



#define degree_180            0
#define degree_150            1
#define degree_120            2
#define degree_90             3
#define degree_60             4
#define degree_30             5
#define degree_0              6
#define degree30              7
#define degree60              8
#define degree90              9
#define degree120             10
#define degree150             11



//Special effects

#define Antique                      0
#define Bluish                       1
#define Greenish                     2
#define Reddish                      3
#define BW                           4
#define Negative                     5
#define BWnegative                   6
#define Normal                       7
#define Sepia                        8
#define Overexposure                 9
#define Solarize                     10
#define  Blueish                     11
#define Yellowish                    12

#define Exposure_17_EV                    0
#define Exposure_13_EV                    1
#define Exposure_10_EV                    2
#define Exposure_07_EV                    3
#define Exposure_03_EV                    4
#define Exposure_default                  5
#define Exposure03_EV                     6
#define Exposure07_EV                     7
#define Exposure10_EV                     8
#define Exposure13_EV                     9
#define Exposure17_EV                     10

#define Auto_Sharpness_default              0
#define Auto_Sharpness1                     1
#define Auto_Sharpness2                     2
#define Manual_Sharpnessoff                 3
#define Manual_Sharpness1                   4
#define Manual_Sharpness2                   5
#define Manual_Sharpness3                   6
#define Manual_Sharpness4                   7
#define Manual_Sharpness5                   8



#define Sharpness1                         0
#define Sharpness2                         1
#define Sharpness3                         2
#define Sharpness4                         3
#define Sharpness5                         4
#define Sharpness6                         5
#define Sharpness7                         6
#define Sharpness8                         7
#define Sharpness_auto                       8




#define EV3                                 0
#define EV2                                 1
#define EV1                                 2
#define EV0                                 3
#define EV_1                                4
#define EV_2                                5
#define EV_3                                6

#define MIRROR                              0
#define FLIP                                1
#define MIRROR_FLIP                         2




#define high_quality                         0
#define default_quality                      1
#define low_quality                          2

#define Color_bar                      0
#define Color_square                   1
#define BW_square                      2
#define DLI                            3


#define Night_Mode_On                  0
#define Night_Mode_Off                 1

#define Off                            0
#define Manual_50HZ                    1
#define Manual_60HZ                    2
#define Auto_Detection                 3



#endif //P2_CAMERA

