//Parallel Camera test code
// RJA captures jpg image from camera after showing a few frames of preview on VGA monitor
//Copyright (C) Raymond Allen 2021
//MIT License

#include "Platform.h"
#include "Cam.h"  //Digital Video Port (DVP) camera driver for P2
struct __using("CameraCapture_JPEG.spin2") Camera2;
//QVGA over VGA driver at 16bpp
struct __using("QVGA_16bpp.spin2") QVGA1;

//Camera DV capture
struct __using("CameraCapture.spin2") Camera1;


i2c_Port I2C1;

int nLoop = 0;

void Setup()
{
	int basepin = 8; //basepin of A/V accessory with VGA
	QVGA1.start(basepin, basepin + 4);

	Camera1.Start(QVGA1.GetBuffer(), DVP1_Basepin, DVP1_VSYNC, DVP1_HSYNC, DVP1_PixelClock, DVP1_Type);

	i2c_Setup(&I2C1, I2C1_SCLPIN, I2C1_SDAPIN, I2C1_SPEED, I2C1_PULLUP); //setup(i2c_Port*, scl, sda, khz, pullup) 
	printf("I2C Driver started.\n");

	//Create NCO clock for camera
	InitCamera(DVP1_Xclk, DVP1_PixelClockFreq, DVP1_RST, DVP1_Type, &I2C1, BMP); //RJA added format option 29Dec22

		//get filesystem ready
#if defined(USE_SD)
	//mount("/files", _vfs_open_sdcard());
	mount("/files", _vfs_open_sdcardx(uSD_CLK, uSD_SS, uSD_MOSI, uSD_MISO));//clock pin, select pin, data in, and data out pins, in that order
	chdir("/files");
#elif defined(USE_HOST)
	mount("/files", _vfs_open_host());
	chdir("/files");
#else
	#warning no file system specified, I hope you are compiling for PC
#endif
}

//Scope measurements 31Dec22:
//in QVGA raw mode, pixel clock is 15 MHz
//In QVGA jpeg mode, pixel clock is 10 MHz, capture data on rising edge
//In VGA jpeg mode, pixel clock is 20 MHz, capture data on rising edge
//in SVGA jpeg mode, pixel clock is 15 MHz, captur data on rising edge
//in XGA jpeg mode, pixel clock is 15 MHz, captur data on rising edge
//In 1600x1200 jpeg mode, pixel clock is 20 MHz, capture data on rising edge

void Loop()
{
	//uint16_t* p = (uint16_t*)QVGA1.GetBuffer();
	ProbeCamera(DVP1_Type, &I2C1);

	printf("Loop #%d\n\n", nLoop++);

	if (nLoop == 5)
	{
		uint8_t* pBuff;
		printf("Capturing jpg...\n");
		Camera1.stop();

		InitCamera(DVP1_Xclk, DVP1_PixelClockFreq, DVP1_RST, DVP1_Type, &I2C1, JPEG); //RJA added format option 29Dec22
		
		pBuff = QVGA1.GetBuffer();
		pBuff[0] = 7;


		Camera2.Start(pBuff, DVP1_Basepin, DVP1_VSYNC, DVP1_HSYNC, DVP1_PixelClock, DVP1_Type);

		printf("Waiting for image capture...\n");
		
		//while (pBuff[0] == 0) {}
		printf("Image capturing...\n");
		_waitms(2000);
		Camera2.Stop();


		

		printf("Image start:\n");
		for (int i = 0; i < 25; i++)
		{
			printf("buffer(%d)=%x\n", i, pBuff[i]);
		}

		printf("Scanning for start:\n");
		int istart = 0;
		for (int i = 0; i < 320*240*2; i++)
		{
			if ((pBuff[i]==0xFF)&& (pBuff[i+1] == 0xD8))
			{
				printf("Image starts at %d\n", i);
				istart = i;
				break;
			}
		}
		
		printf("Scanning for end:\n");
		int iend = -1;
		for (int i = 0; i < (320 * 240*2); i++)
		{
			if ((pBuff[i] == 0xFF) && (pBuff[i + 1] == 0xD9) && (pBuff[i + 2] == 0x00))
			{
				printf("Image ends at %d\n", i+1);
				iend = i+1;
				break;
			}
		}
		if (iend < 0)
		{
			printf("End not found, saving whole buffer.\n");
			iend = 320 * 240 * 2;
		}

						

		// Open the file on SD card by name
		FILE* f = fopen("test.jpg", "wb");
		if (!f)
		{
			printf("%s not open\n", "test.jpg");
			fclose(f);
			for (;;) {}
		}

		int size = iend - istart + 1;

		int w=fwrite(&pBuff[istart], 1, size, f);
		printf("%d bytes written\n",w);
		fclose(f);

		printf("Done.\n");
		for (;;) {}

		
	}
}



int main()
{
 
    char c=0;
    printf("Starting\n");

    Setup();

    for (;;)
    {
        Loop();
        _waitms(1000);
    }


}





